﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Lux_BH1750
{
    public partial class Form1 : Form
    {
        public static byte m_I2cAdr = 0x23 << 1; //I2C-Adresse 7bit -> 8Bit (IO-Warrior arbeitet mit 8 Bit)
        public IntPtr m_handle;
        public int m_Min;
        public int m_Max;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            m_Max = 0;
            m_Min = 0;

            byte[] report = new byte[iow.Defines.IOWKIT_SPECIAL_REPORT_SIZE];    //array für IO-Warrior 24 Special Mode
            StringBuilder sn = new StringBuilder();
            uint pid;

            m_handle = iow.Functions.IowKitOpenDevice();    //Handle für den IO-Warrior holen

            if (m_handle != IntPtr.Zero)
            {
                iow.Functions.IowKitGetSerialNumber(m_handle, sn);  //Seriannummer auslesen
                lbl_sn.Text = sn.ToString();
                pid = iow.Functions.IowKitGetProductId(m_handle);   //Welcher IO-Warrior ist gemeint?

                switch (pid)
                {
                    case iow.Defines.IOWKIT_PID_IOW24:
                        lbl_typ.Text = "IO-Warrior24";
                        break;
                    case iow.Defines.IOWKIT_PID_IOW40:
                        lbl_typ.Text = "IO-Warrior40";
                        break;
                    case iow.Defines.IOWKIT_PID_IOW56:
                        lbl_typ.Text = "IO-Warrior56";
                        break;
                }
                              
                //IO-Warrior24 -> Einschalten des I2C-Modus
                report[0] = 0x01; //Special Mode I2C
                report[1] = 0x01; //Enable
                report[2] = 0x00; //Flags
                report[3] = 0x00; //Timeout
                iow.Functions.IowKitWrite(m_handle, iow.Defines.IOW_PIPE_SPECIAL_MODE, report, 8);

                //Initialisieren des BH1750
                InitBH1750(0x10);
                
                timer1.Start();
            }
            else
                MessageBox.Show("Kein IO-Warrior gefunden", "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            iow.Functions.IowKitCloseDevice(m_handle);  //Nicht vergessen, die DLL beim beenden zu schließen!
        }

        private void InitBH1750(byte value)
        {
            byte[] report = new byte[8];

            report[0] = 0x02;       //I2C Modus für Schreiben
            report[1] = 0xC2;       //Start + Stop + 2 Byte Daten
            report[2] = m_I2cAdr;   //Daten: I2C-Adresse
            report[3] = value;      //Daten: Wert

            if (iow.Functions.IowKitWrite(m_handle, iow.Defines.IOW_PIPE_SPECIAL_MODE, report, 8) == iow.Defines.IOWKIT_SPECIAL_REPORT_SIZE)
            {
                //Antwort des I2C Zurücklesen
                if(iow.Functions.IowKitRead(m_handle, iow.Defines.IOW_PIPE_SPECIAL_MODE, report, 8) == iow.Defines.IOWKIT_SPECIAL_REPORT_SIZE)
                {
                    if((report[1] & 0x80) == 0x80) //Fehler beim Lesen der Daten
                        MessageBox.Show("Fehler beim Initialisieren des BH1750, kein ACK!", "Fehler (Read Daten)", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                {
                    MessageBox.Show("Fehler beim Initialisieren des BH1750", "Fehler (Read)", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
            {
                MessageBox.Show("Fehler beim Initialisieren des BH1750", "Fehler (Write)", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private int ReadLuxData()
        {
            int ret = 0;

            byte[] report = new byte[8];

            report[0] = 0x03;   //I2C Modus für Lesen
            report[1] = 0x02;   // 2 Byte lesen
            report[2] = (byte)(m_I2cAdr | 0x01);    //I2C-Adresse fürs Lesen
            
            if (iow.Functions.IowKitWrite(m_handle, iow.Defines.IOW_PIPE_SPECIAL_MODE, report, 8) == iow.Defines.IOWKIT_SPECIAL_REPORT_SIZE)
            {
                if (iow.Functions.IowKitRead(m_handle, iow.Defines.IOW_PIPE_SPECIAL_MODE, report, 8) == iow.Defines.IOWKIT_SPECIAL_REPORT_SIZE)
                {
                    if ((report[1] & 0x80) == 0x80) //Fehler beim Lesen der Daten
                    {
                        MessageBox.Show("Fehler beim Lesen des BH1750, kein ACK!", "Fehler (Read Daten)", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        ret = -1;
                    }
                    else
                        ret = (int)Math.Round(((report[2] << 8) | report[3]) / 1.2);
                }
                else
                {
                    MessageBox.Show("Fehler beim Lesen des BH1750", "Fehler (Read)", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    ret = -1;
                }
            }
            else
            {
                MessageBox.Show("Fehler beim Lesen des BH1750", "Fehler (Write)", MessageBoxButtons.OK, MessageBoxIcon.Error);
                ret = -1;
            }

            return ret;
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            int val = ReadLuxData();

            //Bei Fehlern Timer stoppen !
            if (val < 0)
                timer1.Stop();

            if (val > m_Max)
            {
                m_Max = val;
                lbl_max.Text = val.ToString() + " Lux";
            }

            if ((val < m_Min) || (m_Min == 0))
            {
                m_Min = val;
                lbl_min.Text = val.ToString() + " Lux";
            }

            lbl_lux.Text = val.ToString() + " Lux";
        }

        private void btn_reset_Click(object sender, EventArgs e)
        {
            m_Min = 0;
            m_Max = 0;
        }
    }
}
